class MessageElement(object):
    pass


class Hr(MessageElement):
    pass


class Markdown(MessageElement):
    content: str

    def __init__(self, content: str):
        self.content = content


class Note(MessageElement):
    content: str

    def __init__(self, content: str):
        self.content = content


class MarkdownAt(MessageElement):
    user_id: str

    def __init__(self, user_id: str):
        self.user_id = user_id


class Confirm(MessageElement):
    title: str
    content: str

    def __init__(self, title: str, content: str):
        self.title = title
        self.content = content


class DatePicker(MessageElement):
    picker_type: str
    picker_id: str
    confirm: Confirm
    placeholder: str

    def __init__(self, picker_type: str, picker_id: str, confirm: Confirm, placeholder: str):
        self.picker_type = picker_type
        self.picker_id = picker_id
        self.confirm = confirm
        self.placeholder = placeholder


class TextAndDatePicker(MessageElement):
    text: str
    date_picker: DatePicker

    def __init__(self, text: str, date_picker: DatePicker):
        self.text = text
        self.date_picker = date_picker


class Button(MessageElement):
    button_text: str
    button_type: str
    button_id: str
    confirm: Confirm

    def __init__(self, button_text: str, button_type: str, button_id: str, confirm: Confirm):
        self.button_text = button_text
        self.button_type = button_type
        self.button_id = button_id
        self.confirm = confirm


class TextAndButton(MessageElement):
    text: str
    button: Button

    def __init__(self, text: str, button: Button):
        self.text = text
        self.button = button


class Option(object):
    text: str
    value: str

    def __init__(self, text: str, value: str):
        self.text = text
        self.value = value


class Options(MessageElement):
    place_holder: str
    option_id: str
    option_type: str
    options: list
    confirm: Confirm

    def __init__(self, place_holder: str, option_id: str, option_type: str, options: list, confirm: Confirm):
        self.place_holder = place_holder
        self.option_id = option_id
        self.option_type = option_type
        self.options = options
        self.confirm = confirm


class TextAndOptions(MessageElement):
    text: str
    options: Options

    def __init__(self, text: str, options: Options):
        self.text = text
        self.options = options



class Message(object):
    pass


class EmptyMessage(Message):
    title: str
    template: str
    elements: list
    msg_type: str

    def __init__(self, title: str, template: str, elements: list, msg_type: str):
        self.title = title
        self.template = template
        self.elements = elements
        self.msg_type = msg_type

    def add_element(self, element):
        self.elements.append(element)


class TextMessage(Message):
    def __init__(self):
        self._title = ""
        self._content = ""
        self._at_all = False

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value):
        self._content = value

    @property
    def at_all(self):
        return self._at_all

    @at_all.setter
    def at_all(self, value):
        self._at_all = value


class MarkdownMessage(Message):

    def __init__(self):
        self._title = ""
        self._content = ""
        self._template = "red"
        self._at_all = False

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value):
        self._content = value

    @property
    def at_all(self):
        return self._at_all

    @at_all.setter
    def at_all(self, value):
        self._at_all = value

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, value):
        self._template = value



import abc

from wiz_message.message import Message


class MessageGenerator(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def to_message(self, message: Message):
        pass


class BotClient(metaclass=abc.ABCMeta):
    def __init__(self, webhook, secret_key):
        self._webhook = webhook
        self._secret_key = secret_key

    @property
    def webhook(self):
        return self._webhook

    @property
    def secret_key(self):
        return self._secret_key

    @classmethod
    @abc.abstractmethod
    def filter(cls, webhook: str):
        pass

    @abc.abstractmethod
    def send_message(self, message: Message):
        pass

    @abc.abstractmethod
    def convert_message(self, message: Message):
        pass


class BotClientHolder(object):
    def __init__(self, abs_client):
        self._clients = []
        self._abs_clients = abs_client

    @property
    def clients(self):
        return self._clients

    def gen_client(self, webhook: str, secret_key: str):
        for item in self._abs_clients:
            if item.filter(webhook):
                self._clients.append(item(webhook, secret_key))
                break

    def send_message(self, message: Message):
        for client in self._clients:
            client.send_message(message)
